grammar SortHDLLang;
   

options {
  k = 2;
}
 
@header {
  	package com.eu.miscedautils.SortHDL;  	 
}

@lexer::header {
 	package com.eu.miscedautils.SortHDL;	 
}

@lexer::members {
	boolean isVerilog = false;
	@Override public void reportError(RecognitionException re)
    {
        return;
    }
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) {
        return;
    }
      @Override
  public void emitErrorMessage(String msg) 
  {
    // For the time being kept empty
  }
}

@members {
	boolean isVerilog = false;
	@Override public void reportError(RecognitionException re)
	{
		return;
    }
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) {
        return;
    }
      @Override
  public void emitErrorMessage(String msg) 
  {
    // For the time being kept empty
  }
}
  
  
@init {
	this.isVerilog = false;
}

// Empty rule

isVerilog returns [boolean vlog]
  :
  module { $vlog = $module.vlog; }
  | udp { $vlog = $udp.vlog; }
  | directive { $vlog = $directive.vlog; }
  |
  (
    SL_COMMENT
    | ML_COMMENT
  )
  { $vlog = true;}
  |
  (
    VHDL_COMMENT
    | ENTITY
    | ARCHITECTURE
    | CONFIGURATION
    | PACKAGE
    | LIBRARY
    | USE
  )
  { $vlog = false;}
  ;

module returns [boolean vlog]
  :
  (
    'module'
    | 'macromodule'
  )
  { $vlog = true;  }
  ;

udp returns [boolean vlog]
  :
  'primitive' { $vlog = true;  }
  ;

directive returns [boolean vlog]
  :
  (
    define_directive
    | include_directive
    | timescale_directive
    | undef_directive
    | resetall_directive
    | conditional_compilation_directive
    | line_compiler_directive
  )
  { $vlog = true;  }
  ;

define_directive
  :
  TICKDEFINE IDENTIFIER
  ;

include_directive
  :
  INCLUDE
  (
    IDENTIFIER
    | STRING
  )
  ;

timescale_directive
  :
  TIMESCALE NUMBER
  (
    'ps'
    | 'ns'
    | 'fs'
    | 'us'
    | 'ms'
//    | 's'
  )
  DIV NUMBER
  (
    'ps'
    | 'ns'
    | 'fs'
    | 'us'
    | 'ms'
//    | 's'
  )
  ;

undef_directive
  :
  UNDEF IDENTIFIER
  ;

resetall_directive
  :
  '`resetall'
  ;

conditional_compilation_directive
  :
  (IFDEF) => ifdef_directive
  | (IFNDEF) => ifndef_directive
  ;

ifdef_directive
  :
  IFDEF IDENTIFIER
  | ELSIF IDENTIFIER
  | ELSE
  | ENDIF
  ;

ifndef_directive
  :
  IFNDEF IDENTIFIER
  ;

line_compiler_directive
  :
  LINE NUMBER STRING
  ;

// Operators

AT
  :
  '@'
  ;

COLON
  :
  ':'
  ;

COMMA
  :
  ','
  ;

DOT
  :
  '.'
  ;

ASSIGN
  :
  '='
  ;

MINUS
  :
  '-'
  ;

LBRACK
  :
  '['
  ;

RBRACK
  :
  ']'
  ;

LCURLY
  :
  '{'
  ;

RCURLY
  :
  '}'
  ;

LPAREN
  :
  '('
  ;

RPAREN
  :
  ')'
  ;

POUND
  :
  '#'
  ;

QUESTION
  :
  '?'
  ;

SEMI
  :
  ';'
  ;

PLUS
  :
  '+'
  ;

LNOT
  :
  '!'
  ;

BNOT
  :
  '~'
  ;

BAND
  :
  '&'
  ;

RNAND
  :
  '~&'
  ;

BOR
  :
  '|'
  ;

RNOR
  :
  '~|'
  ;

BXOR
  :
  '^'
  ;

BXNOR
  :
  '~^'
  | '^~'
  ;

STAR
  :
  '*'
  ;

DIV
  :
  '/'
  ;

MOD
  :
  '%'
  ;

EQUAL
  :
  '=='
  ;

NOT_EQ
  :
  '!='
  ;

NOT_EQ_CASE
  :
  '!=='
  ;

EQ_CASE
  :
  '==='
  ;

LAND
  :
  '&&'
  ;

LOR
  :
  '||'
  ;

LT_
  :
  '<'
  ;

LE
  :
  '<='
  ;

GT
  :
  '>'
  ;

GE
  :
  '>='
  ;

SR
  :
  '>>'
  ;

SL
  :
  '<<'
  ;

TRIGGER
  :
  '->'
  ;

PPATH
  :
  '=>'
  ;

FPATH
  :
  '*>'
  ;

// For VHDL

ENTITY
  :
  (
    'e'
    | 'E'
  )
  (
    'n'
    | 'N'
  )
  (
    't'
    | 'T'
  )
  (
    'i'
    | 'I'
  )
  (
    't'
    | 'T'
  )
  (
    'y'
    | 'Y'
  )
  ;

ARCHITECTURE
  :
  (
    'a'
    | 'A'
  )
  (
    'r'
    | 'R'
  )
  (
    'c'
    | 'C'
  )
  (
    'h'
    | 'H'
  )
  (
    'i'
    | 'I'
  )
  (
    't'
    | 'T'
  )
  (
    'e'
    | 'E'
  )
  (
    'c'
    | 'C'
  )
  (
    't'
    | 'T'
  )
  (
    'u'
    | 'U'
  )
  (
    'r'
    | 'R'
  )
  (
    'e'
    | 'E'
  )
  ;

CONFIGURATION
  :
  (
    'c'
    | 'C'
  )
  (
    'o'
    | 'O'
  )
  (
    'n'
    | 'N'
  )
  (
    'f'
    | 'F'
  )
  (
    'i'
    | 'I'
  )
  (
    'g'
    | 'G'
  )
  (
    'u'
    | 'U'
  )
  (
    'r'
    | 'R'
  )
  (
    'a'
    | 'A'
  )
  (
    't'
    | 'T'
  )
  (
    'i'
    | 'I'
  )
  (
    'o'
    | 'O'
  )
  (
    'n'
    | 'N'
  )
  ;

PACKAGE
  :
  (
    'p'
    | 'P'
  )
  (
    'a'
    | 'A'
  )
  (
    'c'
    | 'C'
  )
  (
    'k'
    | 'K'
  )
  (
    'a'
    | 'A'
  )
  (
    'g'
    | 'G'
  )
  (
    'e'
    | 'E'
  )
  ;

LIBRARY
  :
  (
    'l'
    | 'I'
  )
  (
    'b'
    | 'B'
  )
  (
    'r'
    | 'R'
  )
  (
    'a'
    | 'A'
  )
  (
    'r'
    | 'R'
  )
  (
    'y'
    | 'Y'
  )
  ;

USE
  :
  (
    'u'
    | 'U'
  )
  (
    's'
    | 'S'
  )
  (
    'e'
    | 'E'
  )
  ;
// END VHDL KW

// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the Literals table to see
// if it's a literal or really an identifer.

IDENTIFIER
  //      options {testLiterals=true;}
  :
  (
    'a'..'z'
    | 'A'..'Z'
    | '_'
  )
  (
    'a'..'z'
    | 'A'..'Z'
    | '_'
    | '$'
    | '0'..'9'
  )*
  ;
/*ESCAPED_IDENTIFIER :
        '\\'! (~ '\u0020')+ ('\u0020'|'\t'|'\n')!
        ;*/


ESCAPED_IDENTIFIER
  :
  '\\' (~'\u0020')+
  (
    '\u0020'
    | '\t'
    | '\n'
  )
  ;

SYSTEM_TASK_NAME
  :
  '$' IDENTIFIER
  ;

// string literals

STRING
  :
  '"'
  (
    ~(
      '"'
      | '\n'
     )
  )*
  '"'
  ;

UNDEF
  :
  '`undef'
  ;

IFDEF
  :
  '`ifdef'
  ;

IFNDEF
  :
  '`ifndef'
  ;

ELSIF
  :
  '`elsif'
  ;

TICKDEFINE
  :
  '`define'
  ;

ENDIF
  :
  '`endif'
  ;

ELSE
  :
  '`else'
  ;

LINE
  :
  '`line'
  ;

INCLUDE
  :
  '`include'
  ;

TIMESCALE
  :
  '`timescale'
  ;

// 'compiler' define/macro.

DEFINE
  //options {testLiterals=true;}
  :
  '`' IDENTIFIER
  ;

// a dummy rule to force vocabulary to be all characters (except special
//   ones that ANTLR uses internally (0 to 2)

fragment
VOCAB
  :
  '\u0003'..'\u007F'
  ;

NUMBER
  :
  (
    (
      (SIZE (WS)*)? BASE (WS)*
      (
        (BIT+)
        | (SIZED_DIGIT)
      )
    )
      => SIZED_NUMBER
    | UNSIZED_NUMBER
  )
  ;

// Baya: 1'b1 and 1'b0 does not match. thi is a bug a of today

fragment
SIZED_NUMBER
  :
  (SIZE (WS)*)? BASE (WS)*
  (
    ( (BIT)+)
    |
    (
      SIZED_DIGIT
      (
        SIZED_DIGIT
        | '_'
      )*
    )
  )
  ;

fragment
SIZE
  :
  (DIGIT)+
  ;

fragment
BASE
  :
  '\''
  (
    'd'
    | 'D'
    | 'h'
    | 'H'
    | 'o'
    | 'O'
    | 'b'
    | 'B'
    | 's'
    | 'S'
  )
  ;

fragment
SIZED_DIGIT
  :
  DIGIT
  | BIT
  | HEXDIGIT
  ;

fragment
UNSIZED_NUMBER
  :
  DIGIT
  (
    DIGIT
    | '_'
  )*
  ;

fragment
DIGIT
  :
  ('0'..'9')
  ;

fragment
BIT
  :
  (
    '0'
    | '1'
    | 'x'
    | 'z'
    | 'X'
    | 'Z'
    | '?'
  ) // '0' and '1' are covered in digit
  ;

fragment
HEXDIGIT
  :
  (
    'A'..'F'
    | 'a'..'f'
  )
  ;

fragment
SPACE_OR_TAB
  :
  (
    ' '
    | '\t'
  )+
  ;

WS
  :
  SPACE_OR_TAB+ { $channel=HIDDEN; }
  ;

// The folling rule for newline has been added by Baya as it was not in the orignal grammar file

NEW_LINE
  :
  (
    ('\r' '\n')
    | ('\n')
  )+
  { $channel=HIDDEN; }
  ;

ML_COMMENT
  :
  '/*' (options {greedy=false;}: .)* '*/' { $channel=HIDDEN; isVerilog = true;  }
  ;

SL_COMMENT
  :
  '//' (options {greedy=false;}: .)* '\r'? '\n' {  $channel=HIDDEN; isVerilog = true; }
  ;

VHDL_COMMENT
  :
  '--' (~'\n')* { $channel = HIDDEN;isVerilog  = false;return;}
  ;
